import { BadRequestException, Injectable, NotFoundException } from '@nestjs/common'
import { CreateChallengeActionDto } from './dto/create-challenge-action.dto'
import { UpdateChallengeActionDto } from './dto/update-challenge-action.dto'
import { ChallengeAction, MoneyUpdateType, Player } from '@prisma/client'
import { PrismaService } from 'src/prisma/prisma.service'
import { QueryPaginationDto } from 'src/common/dto/pagination-query.dto'
import { paginate } from 'src/common/utils/pagination.utils'
import { FilterChallengeActionsDto } from './dto/filter-challenge-action.dto'
import { Constants } from 'src/common/utils/constants.utils'

@Injectable()
export class ChallengeActionsService {
  constructor(private prisma: PrismaService) { }

  async create(authenticatedPlayer: Player, createChallengeActionDto: CreateChallengeActionDto): Promise<ChallengeAction> {
    const game = await this.prisma.game.findUnique({ where: { id: 1 } })
    const data = { ...createChallengeActionDto, playerId: authenticatedPlayer.id, runId: game.currentRunId }
    return await this.prisma.challengeAction.create({
      data: data,
    })
  }

  async findAll(queryPagination: QueryPaginationDto, filterChallengeActions: FilterChallengeActionsDto): Promise<[ChallengeAction[], number]> {
    return [
      await this.prisma.challengeAction.findMany({
        ...paginate(queryPagination),
        where: filterChallengeActions,
      }),
      await this.prisma.challengeAction.count(),
    ]
  }

  async findOne(id: number): Promise<ChallengeAction> {
    return await this.prisma.challengeAction.findUnique({
      where: { id },
    })
  }

  async update(id: number, updateChallengeActionDto: UpdateChallengeActionDto): Promise<ChallengeAction> {
    if (!await this.findOne(id))
      throw new NotFoundException(`Aucune action de défi trouvée avec l'identifiant ${id}`)
    return await this.prisma.challengeAction.update({
      where: { id },
      data: updateChallengeActionDto,
    })
  }

  async remove(id: number): Promise<ChallengeAction> {
    if (!await this.findOne(id))
      throw new NotFoundException(`Aucune action de défi trouvée avec l'identifiant ${id}`)
    return await this.prisma.challengeAction.delete({
      where: { id },
    })
  }

  async endCurrentChallenge(player: Player, success: boolean): Promise<ChallengeAction> {
    const challengeAction = await this.prisma.challengeAction.findFirst({
      where: {
        playerId: player.id,
        active: true,
      }
    })
    if (!challengeAction)
      throw new BadRequestException("Aucun défi n'est en cours")
    let data
    const now = new Date()
    if  (success) {
      data = {
        success: success,
        active: false,
        end: now,
      }

      // Crédit du nombre de points remportés grâce au défi
      const challenge = await this.prisma.challenge.findUnique({ where: { id: challengeAction.challengeId } })
      await this.prisma.moneyUpdate.create({
        data: {
          playerId: player.id,
          amount: challenge.reward,
          reason: MoneyUpdateType.WIN_CHALLENGE,
          actionId: challengeAction.id,
        }
      })
    }
    else {
      data = {
        success: success,
        active: false,
        end: now,
        penaltyStart: now,
        penaltyEnd: new Date(now.getTime() + Constants.PENALTY_TIME * 60 * 1000),
      }
    }
    return await this.prisma.challengeAction.update({
      where: {
        id: challengeAction.id,
      },
      data: data,
    })
  }
}
